function [Theta,Eta,Omega]=gtCalcOmega(normal, hkl)

%calculates the Omega position at which diffraction will occur from a plane
%defined by its plane normal and hkl type [h k l]

%load data
parameters=[];
if isempty(parameters)
  load('parameters.mat');
end

%put plane normal in cylindrical polar coords
length = sqrt(normal(1)^2 + normal(2)^2);
alpha = atan2(normal(2), normal(1));
vert = normal(3);

%calc Theta for diffraction
lambda = 12.398/parameters.acq.energy;
if parameters.cryst.spacegroup == 225
d=parameters.cryst.latticepar(1)/sqrt(hkl*hkl');
else
  disp('only works for FCC at the mo...')
end
Theta = asin(lambda/(2*d));

try
  %produce Omega angles
  temp(1) = acos((cos((pi/2)-Theta))/length);
  temp(2) = pi-temp(1);
  Omega = temp - alpha;
  Omega(3:4) = Omega(1:2)+pi;

  %put all in 0-2pi interval
  dummy = find(Omega<0);
  Omega(dummy) = Omega(dummy) + (2*pi);
  dummy = find(Omega>(2*pi));
  Omega(dummy) = Omega(dummy) - (2*pi);

  %calc Eta
for i=1:size(Omega,2)
  alpha2 = alpha + Omega(i);
  y = length*sin(alpha2);
  Eta(i) = sin(y/vert);
  Theta(i) = Theta(1);
end

catch%if no solutions
  disp('no diffraction from this plane normal!')
end




