%
% FUNCTION gtMoveDifspotsIntoSingleTable(parameters,deleteolddata)
%
%  Copies 2 difspot tables into a single one.
%  
% OPTIONAL INPUT
%   parameters: if empty, it will look for the parameters file, otherwise 
%               define the following fields: 
%                 parameters.acq.nproj
%                 parameters.acq.difA_name (without 'difspot' at the end)
%                 parameters.acq.difB_name (without 'difspot' at the end)
%   deletetolddata: if true, old tables will be dropped afterwards. 
%

function gtMoveDifspotsIntoSingleTable(parameters,deletetolddata)

if ~exist('deletetolddata','var')
  deletetolddata=[];
end

if ~exist('parameters','var')
  parameters=[];
end

par_loaded=false;
if isempty(parameters)
  par_loaded=true;
  load('parameters.mat');
end

nproj=parameters.acq.nproj;
difnameA=[parameters.acq.difA_name 'difspot'];
difnameAarch=[parameters.acq.difA_name 'difspot_xxx'];
difnameB=[parameters.acq.difB_name 'difspot'];
difnameBarch=[parameters.acq.difB_name 'difspot_xxx'];
newtablename=parameters.acq.difA_name;


gtDBConnect('admin')

disp(' ')
disp('Processing tables...')
disp('  Values of CentroidImage will be taken from MaxImage.')
disp(' ')

mym(sprintf('DROP TABLE IF EXISTS %s',difnameAarch));
mym(sprintf('DROP TABLE IF EXISTS %s',difnameBarch));

mym(sprintf('RENAME TABLE %s TO %s',difnameA,difnameAarch))
mym(sprintf('RENAME TABLE %s TO %s',difnameB,difnameBarch))

gtDBCreateDifspotTable(newtablename)
newtablename=[newtablename 'difspot'];

disp(sprintf('Copying data from %s...',difnameA))
mysqlcmd=sprintf(['INSERT INTO %s '...
  '(StartImage,EndImage,MaxImage,CentroidImage,ExtStartImage,ExtEndImage,'...
  'Area,CentroidX,CentroidY,BoundingBoxXorigin,'...
  'BoundingBoxYorigin,BoundingBoxXsize,BoundingBoxYsize,Integral) '...
  'SELECT '...
  'StartImage,EndImage,MaxImage,MaxImage,ExtStartImage,ExtEndImage,'...
  'Area,CentroidX,CentroidY,BoundingBoxXorigin,'...
  'BoundingBoxYorigin,BoundingBoxXsize,BoundingBoxYsize,Integral '...
  'FROM %s'],newtablename,...
  difnameAarch);
mym(mysqlcmd);

disp(sprintf('Copying data from %s...',difnameB))
mysqlcmd=sprintf(['INSERT INTO %s '...
  '(StartImage,EndImage,MaxImage,CentroidImage,ExtStartImage,ExtEndImage,'...
  'Area,CentroidX,CentroidY,BoundingBoxXorigin,'...
  'BoundingBoxYorigin,BoundingBoxXsize,BoundingBoxYsize,Integral) '...
  'SELECT '...
  'StartImage+%d,EndImage+%d,MaxImage+%d,MaxImage+%d,ExtStartImage+%d,ExtEndImage+%d,'...
  'Area,CentroidX,CentroidY,BoundingBoxXorigin,'...
  'BoundingBoxYorigin,BoundingBoxXsize,BoundingBoxYsize,Integral '...
  'FROM %s'],newtablename,nproj,nproj,nproj,nproj,nproj,nproj,...
  difnameBarch);
mym(mysqlcmd);

if deletetolddata
  mym(sprintf('DROP TABLE %s',difnameAarch));
  mym(sprintf('DROP TABLE %s',difnameBarch));
  disp(sprintf('Table %s and %s have been dropped.',difnameA,difnameB))
  parameters.acq.difA_name=newtablename;
  parameters.acq.difB_name=[];
  if par_loaded
    save('parameters.mat',parameters)
    disp('Table names in the parameters file have been updated.')
  end
end

disp('Finished.')

end % of function
