function [Theta,Eta,Omega]=gtPredictAngles3D(normal, hkl, parameters)

%calculates the Omega position at which diffraction will occur from a plane
%defined by its plane normal and hkl type [h k l]

%30/10/2006 - modify to follow change in gtConsistancyCheck, and to produce
%only the theta/eta/omega from the plane normal (not its inverse as well)

%load data

if ~exist('parameters','var')
  load('parameters.mat');
end

%put plane normal in cylindrical polar coords
length = sqrt(normal(1)^2 + normal(2)^2);
alpha  = atan2(normal(2), normal(1));
vert   = normal(3);

%calc Theta for diffraction
lambda = 12.398/parameters.acq.energy;

    %%%%%%%%   cubic cases   %%%%%%%%%%%%%
    %EDIT Erik begin
if parameters.cryst.spacegroup == 225 || parameters.cryst.spacegroup == 227 || parameters.cryst.spacegroup == 229 || parameters.cryst.spacegroup == 221
    %EDIT Erik begin
    d=parameters.cryst.latticepar(1)/sqrt(hkl*hkl');
    % modif sabine
    %%%%%%%%   hexagonal cases   %%%%%%%%%%%%%
elseif parameters.cryst.spacegroup == 663 || parameters.cryst.spacegroup == 194 || parameters.cryst.spacegroup == 167 || parameters.cryst.spacegroup == 152
    h=hkl(1);k=hkl(2) ;l=hkl(4);
    d=parameters.cryst.latticepar(1)/(4/3*(h^2+k^2+h*k)+(l*parameters.cryst.latticepar(1)/parameters.cryst.latticepar(3))^2)^0.5;
    % fin modif sabine
else
    warning('unknown spacegroup ! ...aborting');
    return
end

Theta = asin(lambda/(2*d));

%produce Omega angles - find four Omegas (hkl ->2, -h-k-l ->2)
if ((cos((pi/2)-Theta))/length)<=1
    temp(1) = acos((cos((pi/2)-Theta))/length);
    temp(2) = -temp(1);
    Omega = alpha + pi + temp;
	%calc Eta, make Theta to output
	%in our new convention, Eta is measured clockwise from vertical on the
	%images when looking along the beam direction ! 

	alpha2 = alpha - Omega;  %plane normal angles at diffraction

	y = length*sin(alpha2);
	x = length*cos(alpha2);

	Eta = atan2(-y,vert);   %this was y, now -y because of 'looking along beam" sense
	Theta(1:2) = Theta(1); %all Thetas are the same

	%put Omega in 0-2pi interval
	dummy = find(Omega<0);
	Omega(dummy) = Omega(dummy) + (2*pi);
	dummy = find(Omega>(2*pi));
	Omega(dummy) = Omega(dummy) - (2*pi);

	%put in degrees for human users
	Omega = Omega*180/pi;
	Theta = Theta*180/pi;
	Eta   = Eta*180/pi;
else
    disp('no diffraction from this plane normal!')
    Theta=[];
	Eta=[];
	Omega=[];
   
end
  
  




