%predict positions of spots in images
%version2 - designed to work in gtHandleSpot2 sequence, with no
%grain%d_.mat yet written, all information passed as arguments

%Marcelo, Andy, 30/10

function [images,x_direct,y_direct,x_diffr,y_diffr] = gtPredictPositions2(normal, hkl, grainCentroid, plot_flag, parameters)
%take a plane normal, its hkl type (to calc Theta), and a grainCentroid [x,y,z], and returns lists of max images, and x and y
%positions of the ext and dif spots.
%plot_flag=1 plots images

%load data 
if isempty(parameters)
  load('parameters.mat');
end
grainx = grainCentroid(1);
grainy =grainCentroid(2);
zcenter = grainCentroid(3);

%use gtPredictAngles to get Theta, Eta, Omega - note gives a pair of angles
[Theta,Eta,Omega] = gtPredictAngles(normal, hkl);
%convert to radians
Theta=Theta*pi/180;
Eta=Eta*pi/180;
Omega=Omega*pi/180;

%get only Omegas within 0-180 degrees for 180deg data
if parameters.acq.type=='180degree'
  %disp('180 degree data')
  dummy=find(Omega<0 | Omega>pi);
  Theta(dummy)=[];
  Eta(dummy)=[];
  Omega(dummy)=[];
end

%deal with grain position
grain2rot=sqrt((grainx-parameters.acq.bb(3)/2).^2+(grainy-parameters.acq.bb(3)/2).^2);
angle=atan2(parameters.acq.bb(3)/2-grainy,grainx-parameters.acq.bb(3)/2);

%extinction spot position
% y correction with the omega offset in the sample, that is a value to be added to X in the screen
x_direct = parameters.acq.bb(1)+(parameters.acq.bb(3)/2)+(grain2rot.*cos(-Omega+angle)); 
y_direct(1:length(Omega)) = parameters.acq.bb(2)+zcenter;

%diffraction spot position
% radius on the screen (in pixels) with the corrected distance from the sample to the
% detector (y in sample coordinates) with the omega offset
radius = tan(2*Theta).*((parameters.acq.dist/parameters.acq.pixelsize)+(grain2rot*sin(-Omega+angle)));
x_diffr =  x_direct + (radius.*sin(Eta)); % spot position (x position) in the screen 
y_diffr = y_direct - (radius.*cos(Eta)); % vertical coordinate on the screeen

%round to integer values
x_direct = round(x_direct);
y_direct = round(y_direct);
x_diffr = round(x_diffr);
y_diffr = round(y_diffr);

%get full image
images = round((Omega/pi)*parameters.acq.nproj);

  
  
  
  %~~~~~~~~plot results on full images~~~~~~~~~~~~~~~~~~~~~~
  if plot_flag
    for i=1:size(Omega,2)
      
	%convert Omega to image number
  if images(i) >= parameters.acq.nproj
    images(i)=images(i)-parameters.acq.nproj;
    path=sprintf('../%s/',parameters.acq.difB_name);
  else
    path=sprintf('../%s/',parameters.acq.difA_name);
  end
  
	start_image = images(i)-3;
	if start_image<0
		start_image=0;
	end
	end_image = images(i)+3;
	if end_image>parameters.acq.nproj-1;
		end_image=parameters.acq.nproj-1;
	end
	
  start_image
  end_image
	
	full=zeros(parameters.acq.xdet,parameters.acq.ydet);
	for im=start_image:end_image
		full=full+edf_read(sprintf('%s1_preprocessing/full/full%04d.edf',path,im));
	end
	full=full./(end_image-start_image+1);

  figure
	imshow(full,[]);
	hold on;
	plot(x_direct(i),y_direct(i),'og')
	plot(x_diffr(i),y_diffr(i),'ro')
  drawnow
  disp('click image to close and see next spot (if exists...)')
  k=waitforbuttonpress;
    end
  end
  %~~~~~~~~plot results on full images~~~~~~~~~~~~~~~~~~~~~~  
