%predict positions of spots in images

% gtPredictPositions_pair2grain 
%work with difspotpair grain indexing process.  Different grain position
%defination
%360 no a / b half scans

%Marcelo, Andy, 30/10

function [images,x_direct,y_direct,x_diffr,y_diffr] = gtPredictPositions_pair2grain360(normal, hkl, grainCentroid, plot_flag, parameters)
%take a plane normal, its hkl type (to calc Theta), and a grainCentroid [x,y,z], and returns lists of max images, and x and y
%positions of the ext and dif spots.
%plot_flag=1 plots images

%load data 
if isempty(parameters)
  load('parameters.mat');
end


%use gtPredictAngles to get Theta, Eta, Omega - note gives a pair of angles
[Theta,Eta,Omega] = gtPredictAngles(normal, hkl);
%returns in degree

%get only Omegas within 0-180 degrees for 180deg data
if parameters.acq.type=='180degree'
  %disp('180 degree data')
  dummy=find(Omega<0 | Omega>180);
  Theta(dummy)=[];
  Eta(dummy)=[];
  Omega(dummy)=[];
end



  
%deal with grain position
[grainCentroid_om] = sfSample_to_setup_cor(grainCentroid, Omega');

%extinction spot position
x_direct = parameters.acq.rotx + grainCentroid_om(:,2); % y coord of grainpos corresponds to x on detector
y_direct = parameters.acq.bb(2)-grainCentroid_om(:,3); 
%z in relative to the top to the sample bb, positive z moves spot towards top of image, hence negative

%diffraction spot position
radius = tand(2*Theta').*((parameters.acq.dist/parameters.acq.pixelsize)-grainCentroid_om(:,1));
x_diffr =  x_direct + (radius.*sind(Eta')); % spot position (x position) in the screen 
y_diffr = y_direct - (radius.*cosd(Eta')); % vertical coordinate on the screeen

%round to integer values
x_direct = round(x_direct);
y_direct = round(y_direct);
x_diffr = round(x_diffr);
y_diffr = round(y_diffr);

%get full image
images = round((Omega'/180)*parameters.acq.nproj);

  


  
  
  function [insetup_vec]=sfSample_to_setup_cor(insample_vec,om)
    % om = omega angle in degrees
    %insetup_vec=([cosd(om) sind(om) 0; -sind(om) cosd(om) 0; 0 0
    %1]*insample_vec')' ;
    insetup_vec(:,1)= cosd(om)*insample_vec(1)+sind(om)*insample_vec(2) ;
    insetup_vec(:,2)= -sind(om)*insample_vec(1)+cosd(om)*insample_vec(2) ;
    insetup_vec(:,3)= insample_vec(3) ;
  end


end
  
  
  
