function gtUpdateDifspotGrainIDs(table, variable, type, lastcolumn, queryname, queryvalues, values)
% GTUPDATEDIFSPOTIDS   Updates table's column with values adding a new column
%     gtUpdateDifspotGrainIDs(table, variable, type, lastcolumn, queryname, queryvalues, values)
%     ------------------------------------------------------------------------------------------
%     Adds/updates values to a specific set of rows in a table in the db
%
%     Usage:
%       for ii=1:length(dir('4_grains/phase_01/grain_*.mat'))
%           gr{ii} = load(sprintf('4_grains/phase_01/grain_%04d.mat',ii),'difspotID');
%       end
%       difspotID = gtIndexAllGrainValues(gr, 'difspotID',[],[],[],false);
%       gtUpdateDifspotGrainIDs([parameters.acq.name 'difspot'], 'grainID', 'int', 'integral', 'difspotID', difspotID, 1:length(gr))
%
%     Version 002 08-11-2013 by LNervo


% if it doesn't already exist, we may need to add a grainid column to
% difspot table

[a,~,~,~,~,~]=mym(['SHOW COLUMNS FROM ' table ' LIKE "' variable '"']);
if isempty(a)
  %add the field
  mym(['ALTER TABLE graintracking.' table ' ADD COLUMN ' variable ' ' type ' AFTER ' lastcolumn]);
  disp(['adding column ' variable ' to the ' table ' table'])
end



% then loop through, updating according to the grain structure information
for ii=1:length(queryvalues)
  fprintf('done %04d', values(ii))
  ids = queryvalues{ii};
  for jj=1:length(ids)
      mysqlcmd = gtDBUpdate(table, queryname, ids(jj), variable, values(ii));
      mym(mysqlcmd);
  end
  fprintf('\b\b\b\b\b\b\b\b\b\b')
end
disp(' ')
disp(['updated column ' variable ' from ' table ' table'])

end % end of function
