function [pof, good_plnorms, dsqr]=plot_rodrigues_consistancy(plane_normal,hkl,spacegroup,plot_flag)
%plane_normal = list of plane normals in sample coords
%hkl a vector of reflection type eg [1 1 1; 0 0 2; 2 2 0 etc]
%plot_flag: 1 to plot figure, 0 to not plot anything
%add a consistancy check - which plane normals don't join at the
%intersection?

inc=0.1;                % scan space in these steps (size of the sampling element) for coarse solution
inc_ext=1.1*inc;         % extended space to determine precise point of intersection; (should be larger than 0.87*inc)
consistancy_test=0.2;   %select those lines within distance consistancy_test*inc of the intersection (was 0.2)

save_plot=[];

if plot_flag
figure;
hold on;
end

lines.cor=[];   %[...; r_zero r_direction; ...] equations
lines.plnorm=[];   %[...; j; ...] which plane normal

for j=1:size(plane_normal,1) % loop of plane_normals

%________calculate h vector___________
%from eq (3.7) in Poulsen
%B is identity matrix in cubic material
%gtnewGetReflections returns matrix of all reflections related by symmetry
%to hkl, using Riso GetCubicSymOp.m

hkl_reflections = gtCrystSignedHKLs(hkl(j,:), gtCrystGetSymmetryOperators([], spacegroup) );
if size(hkl_reflections,1)~=1%skip if hkl = [0 0 0]
  
for i=1:length(hkl_reflections) % loop of hkl reflections
h = hkl_reflections(i,:); 
h = h./(sqrt(h*h'));

%________calculate y vector___________
% y is the normalised scattering vector in sample system
% == plane normal
y = plane_normal(j,:);


%_____calculate rodrigues space vector_____
%formulae (3.16), (3.17) in Poulsen

r_zero = cross(h,y)./(1+(dot(h,y)));
r_direction = (h + y)./(1+(dot(h,y)));

%collect plot data
lines.cor=[lines.cor; r_zero r_direction];
lines.plnorm=[lines.plnorm; j];

%max line length needed ~1.5, say +-1.5 for saftey
t=(-1.5:0.01:1.5)';
r_plot = repmat(r_zero,length(t),1) + (t*r_direction);

%remove everything outside of a fundamental region (assume cube)
lim=sqrt(2)-1;
for i=1:3
dummy=find(r_plot(:,i)<-lim | r_plot(:,i)>lim);
r_plot(dummy,:)=[];
end

%remove the corners of the truncated cube
lim=1/sqrt(3);
%calculate distance of each point along each <111> directions
vector=[1 1 1]./sqrt(3);%normalise
vector=repmat(vector,size(r_plot,1),1);
tmp = sum( ((abs(r_plot)).*vector),2); %projections along <111>
dummy=find(tmp>lim);
length(dummy);
r_plot(dummy,:)=[];


%collect all plot data for analysis
save_plot=[save_plot; r_plot];


%____plot______
if plot_flag
  if all(hkl(j,:)==[1 1 1]) %(111) - red
  plot3(r_plot(:,1),r_plot(:,2),r_plot(:,3),'r');
  elseif all(hkl(j,:)==[1 1 0]) %(110) - cyan - BCC
  plot3(r_plot(:,1),r_plot(:,2),r_plot(:,3),'c');
  elseif all(hkl(j,:)==[0 0 2]) %(002) - blue
  plot3(r_plot(:,1),r_plot(:,2),r_plot(:,3),'b');
  elseif all(hkl(j,:)==[2 1 1]) %(110) - magenta - BCC
  plot3(r_plot(:,1),r_plot(:,2),r_plot(:,3),'m');
  elseif all(hkl(j,:)==[2 2 0]) %(220)  - green
  plot3(r_plot(:,1),r_plot(:,2),r_plot(:,3),'g');
  elseif all(hkl(j,:)==[3 1 1]) %(311)   -  black
  plot3(r_plot(:,1),r_plot(:,2),r_plot(:,3),'k');
  end
end

end%loop through all hkl variants
end%if hkl is okay (skip hkl = [0 0 0])

%uncomment this to see line by line in rodrigues space.
%k=waitforbuttonpress %can watch as each is plotted in turn

end%loop through all normals

if plot_flag
axis equal
end


%find intersections
lim=sqrt(2)-1;
max_score=0;
max_coords=[0 0 0];
score=[];
count=1;
for i=-lim:inc:lim
for j=-lim:inc:lim
for k=-lim:inc:lim
  
dummy = find(save_plot(:,1)>i & save_plot(:,1)<(i+inc) &save_plot(:,2)>j & save_plot(:,2)<(j+inc) &save_plot(:,3)>k & save_plot(:,3)<(k+inc));
score(count)=length(dummy);
if (score(count)>max_score)
  max_score=score;
  max_coords=[i j k];
end

count=count+1;
end
end
end

element = inc*[0 0 0; 0 0 1; 0 1 0; 0 1 1; 1 0 0; 1 0 1; 1 1 0; 1 1 1];
max_element = repmat(max_coords,8,1)+element;
max_coords = max_coords+0.5*[inc inc inc]; %this is the approximate solution




if plot_flag
  plot3(max_element(:,1),max_element(:,2),max_element(:,3),'go')
  plot3(max_coords(:,1),max_coords(:,2),max_coords(:,3),'co')
end

%Peter's bit - more precise handling of intersections
% Determine precise point of intersection inside the sampling element

%normalise the r_direction part of the line equations
%note that it is necessary to normalise vectors for the distance calculations to
%work!
lines.cor=normlinedir(lines.cor);

line_cand.cor=[];%line equations which pass close to max_coords
line_cand.plnorm=[];%to which plane normal (struct_id entry) does line belong - to ensure only one line from each normal

% pick lines which pass close to the sampling element: candidate lines
distances=pointtolinedist(max_coords,lines.cor);
dummy=find(distances< inc_ext);
line_cand.cor = lines.cor(dummy,:);
line_cand.plnorm = lines.plnorm(dummy);


% If more than one line from a given plane is present in the set, then choose the one that fits the best.
pof_prel=pofintersectexp(line_cand.cor)'; %initial least squares best intersection

% Alternative (cleaner!) loop to find the closest line from a given plane
% to the point of intersection, if more than one is present
  distances=pointtolinedist(pof_prel, line_cand.cor);%distance from each candidate line to point
dummy2=[];%list of lines to delete
for k=1:size(plane_normal,1)
  dummy=find(plane_normal==k);
  if length(dummy)>2%if more than one line, delete all but the closest
    %- make a list of those to delete
    dummy2 = [dummy2 find(distances~=min(distances(dummy)) & line_cand.plnorm ==k)];
  end
end
line_cand.cor(dummy2,:)=[];%do the deletion
line_cand.plnorm(dummy2,:)=[];%do the deletion

%line_cand now contains information only about lines which pass near the
%approx intersection, and only the best line arising from each plane
%normal

%repeat loop, to try and choose more precisely
pof_prel=pofintersectexp(line_cand.cor)'; %least squares best intersection

%find distances of remaining lines to point
distances=pointtolinedist(pof_prel, line_cand.cor);%distance from each candidate line to point
test_distance = consistancy_test*inc;
dummy2=find(distances>test_distance);

%%display the distances and which plane normals they come from
%distances
%line_cand.plnorm

line_cand.cor(dummy2,:)=[];%do the deletion
line_cand.plnorm(dummy2,:)=[];%do the deletion


%%[line_cand.cor,line_cand.plnorm]

%final least squares fit with good lines only, display results
pof=pofintersectexp(line_cand.cor)';
dsqr=pointtolinedistsqrsum(max_coords, line_cand.cor);

%consistancy check part
good_plnorms = 1:size(plane_normal,1);%index of plane normals
good_plnorms = ismember(good_plnorms,line_cand.plnorm);


if plot_flag
  plot3(pof(1),pof(2),pof(3),'mo','markersize',25);
  plot3(pof(1),pof(2),pof(3),'m*','markersize',25);
end

